@@ -1,5 +1,5 @@
 //
-// Copyright 2007 Johannes Hofmann <Johannes.Hofmann@gmx.de>
+// Copyright 2007-2009 Johannes Hofmann <Johannes.Hofmann@gmx.de>
 //
 // This software may be used and distributed according to the terms
 // of the GNU General Public License, incorporated herein by reference.
@@ -255,7 +255,7 @@ int PSEditModel::load(FILE *fp) {
 	while (fgets(linebuf, sizeof(linebuf), fp) != NULL) {
 		if (!p2->parse(linebuf)) {
 			ret = write(tmp_fd, linebuf, strlen(linebuf));
-			if (ret != strlen(linebuf)) {
+			if (ret != (int) strlen(linebuf)) {
 				fprintf(stderr, "Error while writing to temporary file\n");
 			}
 		}
