@@ -1,5 +1,5 @@
 //
-// Copyright 2007 Johannes Hofmann <Johannes.Hofmann@gmx.de>
+// Copyright 2007-2009 Johannes Hofmann <Johannes.Hofmann@gmx.de>
 //
 // This software may be used and distributed according to the terms
 // of the GNU General Public License, incorporated herein by reference.
@@ -35,8 +35,7 @@ PostscriptDSC::parse(int fd) {
 	FILE *fp;
 	char linebuf[1024];
 	int x, y, w, h;
-	int p1 = 0, ps;
-	int i = 0;
+	int p1 = 0;
 	int bb_read = 0;
 	int max_pages = 128;
 
