@@ -1,5 +1,5 @@
 //
-// Copyright 2006-2008 Johannes Hofmann <Johannes.Hofmann@gmx.de>
+// Copyright 2006-2009 Johannes Hofmann <Johannes.Hofmann@gmx.de>
 //
 // This software may be used and distributed according to the terms
 // of the GNU General Public License, incorporated herein by reference.
@@ -25,6 +25,8 @@
 #include <FL/Fl_Color_Chooser.H>
 #include <FL/Fl_Hold_Browser.H>
 
+#include <assert.h>
+
 #include "PSEditor.H"
 #include "util.h"
 #include "../config.h"
@@ -68,14 +70,44 @@ confirm_overwrite(const char *f) {
 		return 1;
 }
 
-char filename[1024] = "";
+char *filename = NULL;
+
+
+void scroll_to_begin() {
+	assert(scroll);
+
+	scroll->position(0, 0);
+	scroll->redraw();
+}
+
+
+void scroll_to_end() {
+	assert(scroll && psed_p);
+
+	scroll->position(0, psed_p->h() - scroll->h());
+	scroll->redraw();
+}
+
 
 void page_sel_cb(Fl_Widget *w, void *) {
-	int p = page_sel->value();
-	if (p > 0)
+	int new_page, old_page;
+
+	assert(page_sel && psed_p);
+
+	new_page = page_sel->value();
+	old_page = psed_p->get_page();
+
+	if (new_page > 0 && new_page != old_page) {
 		psed_p->load_page(page_sel->value());
+
+		if(old_page < new_page)
+			scroll_to_begin();
+		else if(old_page > new_page)
+			scroll_to_end();
+	}
 }
 
+
 void page_sel_fill() {
 	char buf[128];
 	int p = psed_p->get_pages();
@@ -139,8 +171,12 @@ void open_cb() {
 		return;
 
 	file = fl_file_chooser("Open File?", "*.{ps,pdf}", filename);
-	if (file)
+	if (file) {
 		open_file(file);
+		if (filename)
+			free(filename);
+		filename = strdup(file);
+	}
 }
 
 void export_pdf_cb() {
@@ -150,7 +186,7 @@ void export_pdf_cb() {
 	char *args[32];
 	pid_t pid;
 
-	file = fl_file_chooser("Open File?", "*.pdf", filename);
+	file = fl_file_chooser("Open File?", "*.pdf", NULL);
 	if(file != NULL && confirm_overwrite(file)) {
 		args[0] = "ps2pdf";
 		args[1] = "-";
@@ -181,7 +217,7 @@ void export_pdf_cb() {
 }
 
 void import_cb() {
-	char *file = fl_file_chooser("Import Overlay from File?", "*.ps", filename);
+	char *file = fl_file_chooser("Import Overlay from File?", "*.ps", NULL);
 	if (file)
 		psed_p->import(file);
 }
@@ -189,13 +225,48 @@ void import_cb() {
 void first_cb() {
 	psed_p->load();
 	page_sel->select(psed_p->get_page());
+	scroll_to_begin();
 }
 
-void next_cb() {
-	psed_p->next();
+
+void last_cb() {
+	int pg = psed_p->get_pages();
+
+	if(psed_p->get_page() == pg)
+		return;
+
+	psed_p->load_page(pg);
 	page_sel->select(psed_p->get_page());
+	scroll_to_begin();
 }
 
+
+void next_cb() {
+	int pg = psed_p->get_page();
+
+	psed_p->next();
+
+	if(psed_p->get_page() == pg)
+		return;
+
+	page_sel->select(psed_p->get_page());
+	scroll_to_begin();
+}
+
+
+void prev_cb() {
+	int pg = psed_p->get_page();
+
+	psed_p->prev();
+
+	if(psed_p->get_page() == pg)
+		return;
+
+	page_sel->select(psed_p->get_page());
+	scroll_to_end();
+}
+
+
 void quit_cb() {
 	if (!check_save())
 		return;
@@ -240,7 +311,7 @@ void print_cb() {
 void about_cb() {
 	fl_message("flpsed -- a PostScript annotator\n"
 		"Version %s\n\n"
-		"(c) Johannes Hofmann 2004-2008\n\n"
+		"(c) Johannes Hofmann 2004-2009\n\n"
 		"PostScript is a registered trademark of Adobe Systems", VERSION);
 }
 
@@ -310,10 +381,7 @@ void zoom_cb(Fl_Widget *w, void *) {
 	if (m) {
 		if (psed_p) {
 			psed_p->zoom(atoi(m->label()));
-			if (scroll) {
-				scroll->position(0,0);
-				scroll->redraw();
-			}
+			scroll_to_begin();
 		}
 	}
 }
@@ -348,7 +416,9 @@ Fl_Menu_Item menuitems[] = {
 
 	{ "&Page", 0, 0, 0, FL_SUBMENU },
 	{ "&First",        FL_CTRL + 'f', (Fl_Callback *)first_cb },
+	{ "&Last",        FL_CTRL + 'l', (Fl_Callback *)last_cb },
 	{ "&Next",       FL_CTRL + 'n', (Fl_Callback *)next_cb },
+	{ "P&revious",       FL_CTRL + 'r', (Fl_Callback *)prev_cb },
 	{ 0 },
 
 	{ "&Zoom", 0, 0, 0, FL_SUBMENU },
@@ -571,8 +641,10 @@ int main(int argc, char** argv) {
 		if (zoom_val)
 			psed_p->zoom(zoom_val);
 
-		if (my_argc >= 1)
-			open_file(my_argv[0]);
+		if (my_argc >= 1) {
+			filename = strdup(my_argv[0]);
+			open_file(filename);
+		}
 
 		for (int i = 0; i < tv_idx; i++) {
 			psed_p->replace_tag(tv[i].tag, tv[i].value);
